/* ************************************************************************

   Copyright:

   License:

   Authors:

************************************************************************ */

/* ************************************************************************

#resource(image:image)

************************************************************************ */


var plasma_entities = new Array(
    {
        name      : "Gyrofrequency",
        dimension : "frequency",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getGyroFrequency(this.B)"
    },
    {
        name      : "Plasma frequency",
        dimension : "frequency",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getPlasmaFrequency()"
    },

    {
        name      : "Debye length",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getDebyeLength()"
    },
    {
        name      : "Full Debye length",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "this.getFullDebyeLength(%s1)"
    },
    {
        name      : "deBroglie length",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getDeBroglieLength()"
    },
    {
        name      : "Mean interparticle distance",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getMeanDistance()"
    },
    {
        name      : "Distance of minimal approach",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getMinApproachDistance()"
    },
    {
        name      : "Gyroradius",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getGyroRadius(this.B)"
    },
    {
        name      : "Inertial length",
        dimension : "length",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getInertialLength()"
    },

    {
        name      : "Thermal velocity",
        dimension : "velocity",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getThermalVelocity()"
    },
    {
        name      : "Alfven velocity",
        dimension : "velocity",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getAlfvenVelocity(this.B)"
    },

    {
        name      : "Coupling parameter",
        dimension : "none",
        nspecies  : 1,
        section   : "fundamental",
        formula   : "%s1.getCoupling()"
    },
    {
        name      : "Typical Debye screening",
        dimension : "none",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "var r = %s1.getMeanDistance()/%s1.getDebyeLength(); \
                     (1 + r)*Math.exp(-r)"
    },

    {
        name      : "Transition energy",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "this.getWavenumber()"
    },
    {
        name      : "Natural linewidth",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "this.getNaturalWidth()"
    },
    {
        name      : "Doppler FWHM",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "2*this.getDopplerHwhm()"
    },
    {
        name      : "Zeeman splitting",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "2*this.getZeemanSplitting()"
    },
    {
        name      : "Stark FWHM",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "2*this.getStarkHwhm()"
    },
    {
        name      : "QS Stark FWHM",
        dimension : "energy",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "2*this.getQsHwhm(%s1)"
    },

    {
        name      : "Dynamic Stark range",
        dimension : "energy",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "this.getMicrofieldFrequency(%s1)/3e10/(2*Math.PI)"
    },

    {
        name      : "Minimal energy distance",
        dimension : "energy",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "this.getMinEnergyDistance()"
    },

    {
        name      : "Static/dynamic Stark ratio",
        dimension : "none",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "this.getStarkRatio(%s1)"
    },
    {
        name      : "Stark quasistaticity",
        dimension : "none",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "this.getStarkQuasistaticity(%s1)"
    },

    {
        name      : "Transition wavelength",
        dimension : "length",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "1/this.getWavenumber()"
    },

    {
        name      : "Transition frequency",
        dimension : "frequency",
        nspecies  : 0,
        section   : "spectroscopy",
        formula   : "3e10*this.getWavenumber()"
    },

    {
        name      : "Holtsmark field",
        dimension : "efield",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "%s1.getHoltsmarkField()"
    },

    {
        name      : "Microfield frequency",
        dimension : "frequency",
        nspecies  : 1,
        section   : "spectroscopy",
        formula   : "this.getMicrofieldFrequency(%s1)"
    }
);


function LymanA(n)
{
    if (n <= 1) {
        return 0;
    } else {
        return 8e9*256/9*n*Math.pow((n - 1)/(n + 1), 2*n)/((n*n - 1)*(n*n - 1));
    }
}


function NumPrint(v)
{
    var s;
    if (v == 0.0) {
        s = "0";
    } else
    if (!isFinite(v)) {
        s = v;
    } else
    if (Math.abs(v) < 1.e6 && Math.abs(v) >= 0.1) {
        s = sprintf("%.3f", v);
    } else {
        s = sprintf("%.3e", v);
    }
    
    return s;
}

function changeN_e(ev)
{
    var e = ev.getTarget();
    var N_e = e.getNumValue();
    
    if (isFinite(N_e) && this.e.setN(N_e)) {
        this.updatePlasmaParameters();
        this.updateGUI();
    }
}

function changeT_e(ev)
{
    var e = ev.getTarget();
    var T_e = e.getNumValue();
    
    if (isFinite(T_e)) {
        this.e.setT(T_e);
        this.i.setT(T_e);
        this.r.setT(T_e);
        this.updateGUI();
    }
}

function changeZ_i(ev)
{
    var e = ev.getTarget();
    var Z_i = parseInt(e.getValue());
    
    if (isFinite(Z_i)) {
        this.i.setQ(Z_i);
        this.updatePlasmaParameters();
        this.updateGUI();
    }
}

function changeT_i(ev)
{
    var e = ev.getTarget();
    var T_i = e.getNumValue();
    
    if (isFinite(T_i)) {
        this.i.setT(T_i);
        this.r.setT(T_i);
        this.updateGUI();
    }
}

function changeZ_r(ev)
{
    var e = ev.getTarget();
    var Z_r = parseInt(e.getValue());
    
    if (isFinite(Z_r)) {
        this.r.setQ(Z_r);
        this.updatePlasmaParameters();
        this.updateGUI();
    }
}

function changeP_r(ev)
{
    var e = ev.getTarget();
    var P_r = parseInt(e.getValue())/100.0;
    
    if (isFinite(P_r)) {
        this.P_r = P_r;
        this.updatePlasmaParameters();
        this.updateGUI();
    }
}

function changeT_r(ev)
{
    var e = ev.getTarget();
    var T_r = e.getNumValue();
    
    if (isFinite(T_r)) {
        this.r.setT(T_r);
        this.updateGUI();
    }
}

function changeM_i(ev)
{
    var e = ev.getTarget();
    var M_i = parseInt(e.getValue());
    
    if (isFinite(M_i)) {
        this.i.setM(M_i);
        this.updateGUI();
    }
}

function changeM_r(ev)
{
    var e = ev.getTarget();
    var M_r = parseInt(e.getValue());
    
    if (isFinite(M_r)) {
        this.r.setM(M_r);
        this.updateGUI();
    }
}

function changeN_u(ev)
{
    var e = ev.getTarget();
    var n_u = parseInt(e.getValue());
    
    this.n_u = n_u;
    this.updateGUI();
}

function changeN_l(ev)
{
    var e = ev.getTarget();
    var n_l = parseInt(e.getValue());
    
    this.n_l = n_l;
    this.updateGUI();
}

function changeB(ev)
{
    var e = ev.getTarget();
    var v = e.getNumValue();
    
    if (isFinite(v)) {
        this.B = v;
    }
}

function treeMenuCB(ev)
{
    if (ev.getButton() == "right") {
        var menu = this.treeMenu;

        if (menu.isSeeable()) {
            menu.hide();
        } else {
            var leaf = ev.getTarget();
            var favorite = leaf.getUserData("favorite");
            var el = leaf.getElement();
            
            if (favorite) {
                this.treeMenuButtonAddToFav.setEnabled(false);
                this.treeMenuButtonRemoveFromFav.setEnabled(true);
            } else {
                this.treeMenuButtonAddToFav.setEnabled(true);
                this.treeMenuButtonRemoveFromFav.setEnabled(false);
            }
            
            menu.setUserData("leaf", leaf);

            // position the menu
            menu.setLeft(qx.html.Location.getPageBoxLeft(el));
            menu.setTop(qx.html.Location.getPageBoxBottom(el));

            menu.show();
        };

        ev.setPropagationStopped(true);
    }
}

function showFundamentalCB(ev)
{
    this.showFundamental = !this.showFundamental;
    this.rePopulateTree();
}

function showSpectroscopyCB(ev)
{
    this.showSpectroscopy = !this.showSpectroscopy;
    this.rePopulateTree();
}

function showElectronsCB(ev)
{
    this.showElectrons = !this.showElectrons;
    this.rePopulateTree();
}

function showIonsCB(ev)
{
    this.showIons = !this.showIons;
    this.rePopulateTree();
}

function showRadiatorsCB(ev)
{
    this.showRadiators = !this.showRadiators;
    this.rePopulateTree();
}

function addToFavCB(ev)
{
    var leaf = this.treeMenu.getUserData("leaf");
    leaf.setUserData("favorite", true);
    leaf.setTextColor("red");
}

function removeFromFavCB(ev)
{
    var leaf = this.treeMenu.getUserData("leaf");
    leaf.setUserData("favorite", false);
    leaf.setTextColor("black");
}

function clickEntityCB(ev)
{
    var e = ev.getTarget();
    var entity = e.getUserData("entity");
    
    // keep the old value of the units
    var combo = this.gui_entity_units;
    
    var old_dimension = combo.getUserData("dimension");
    if (old_dimension != entity.dimension) {
        var list = combo.getList();
        
        // clear everything
        list.removeAll();

        var selected;

        switch (entity.dimension) {
        case "frequency":
                       combo.addNumItem("Rad/s", 2*Math.PI);
            selected = combo.addNumItem("Hz",    1.0);
            break;
        case "length":
                       combo.addNumItem("A",  1.0e8);
                       combo.addNumItem("nm", 1.0e7);
            selected = combo.addNumItem("cm", 1.0);
                       combo.addNumItem("m",  1.0e-2);
            break;
        case "velocity":
            selected = combo.addNumItem("cm/s", 1.0);
                       combo.addNumItem("m/s",  1.0e-2);
                       combo.addNumItem("v/c",  1.0/3e10);
            break;
        case "energy":
            selected = combo.addNumItem("cm^-1", 1.0);
                       combo.addNumItem("eV",    1.0/8065.5);
                       combo.addNumItem("erg",   1.9864e-16);
            break;
        case "efield":
                       combo.addNumItem("V/m",      1.0e2);
            selected = combo.addNumItem("V/cm",     1.0);
                       combo.addNumItem("StatV/cm", 1.0/300);
                       combo.addNumItem("MV/cm",    1.0e-6);
            break;
        case "none":
            selected = combo.addNumItem("---", 1.0);
            break;
        default:
            selected = combo.addNumItem("???", 1.0);
            break;
        }

        if (selected) {
            combo.setSelected(selected);
        }
    }

    combo.setUserData("dimension", entity.dimension);
    
    this.updateEntityValue();
}

function changeUnitsCB(ev)
{
    this.updateEntityValue();
}

/**
 * The application
 */
qx.Class.define("pf.Application",
{
    extend : qx.application.Gui,

    /*
    *****************************************************************************
       MEMBERS
    *****************************************************************************
    */

    members :
    {
        R0 : 0.5, // 0.3/0.7/1 ?
        
        showFundamental  : true,
        showSpectroscopy : true,

        showElectrons  : true,
        showIons       : true,
        showRadiators  : true,

        checkPlasmaParameters : function()
        {
            if ((this.P_r < 0.0 || this.P_r > 1.0) ||
                this.Z_i < 1 ||
                this.P_r == 1.0 && this.Z_r == 0) {
                return false;
            } else {
                return true;
            }
        },

        updatePlasmaParameters : function()
        {
            // Safety measures
            if (this.checkPlasmaParameters() != true) {
                return false;
            }
            
            var N_e = this.e.getN();
            var N_i, N_r;
            
            if (this.P_r < 1.0) {
                N_i = N_e*(1.0 - this.P_r)/
                    (this.i.getQ()*(1.0 - this.P_r) + this.r.getQ()*this.P_r);
                
                N_r = N_i*this.P_r/(1.0 - this.P_r);
            } else {
                N_i = 0.0;
                N_r = N_e/this.r.getQ();
            }
            
            this.i.setN(N_i);
            this.r.setN(N_r);
            
            return true;
        },
        
        updateGUI : function()
        {
            this.gui_N_i.setNumValue(this.i.getN());
            this.gui_N_r.setNumValue(this.r.getN());
            this.gui_T_i.setNumValue(this.i.getT());
            this.gui_T_r.setNumValue(this.r.getT());
            this.gui_Zcore.setValue(this.r.getQ() + 1);
        },
        
        getWavenumber : function()
        {
            var n_u = this.n_u;
            var n_l = this.n_l;
            var Z_core = this.r.getQ() + 1;
            
            return pf.base.Bohr.transitionEnergy(Z_core, n_u, n_l);
        },
        
        getMinEnergyDistance : function()
        {
            var n;
            if (this.n_u < this.n_l) {
                n = this.n_l;
            } else {
                n = this.n_u;
            }
            
            var Z_core = this.r.getQ() + 1;
            
            return pf.base.Bohr.transitionEnergy(Z_core, n + 1, n);
        },
        
        getNaturalWidth : function()
        {
            var n_u = this.n_u;
            var n_l = this.n_l;
            var Z_core = this.r.getQ() + 1;
            
            return Math.pow(Z_core, 4)*
                (LymanA(n_u) + LymanA(n_l))/3e10/(2*Math.PI);
        },
        
        getDopplerHwhm : function()
        {
            var v   = this.r.getThermalVelocity();
            var nu  = this.getWavenumber();
            
            return Math.sqrt(2*Math.log(2))*nu*v/3e10;
        },
        
        getMicrofieldFrequency : function(s)
        {
            var r = s.getMeanDistance();
            var v_r = this.r.getThermalVelocity();
            var v_s = s.getThermalVelocity();
            var v = Math.sqrt(v_r*v_r + v_s*v_s);
            
            return v/r;
        },
        
        getFullDebyeLength : function(s)
        {
            var tmp = 0;
            var m = s.getM();
            
            var sa = new Array(this.e, this.i, this.r);
            for (var i = 0; i < 3; i++) {
                var os = sa[i];
                if (os.getM() <= m) {
                    var ld = os.getDebyeLength();
                    if (ld) {
                        tmp += 1/(ld*ld);
                    }
                }
                
            }
            
            return 1/Math.sqrt(tmp);
        },
        
        getPpiFactor : function(s)
        {
            var dl = this.getFullDebyeLength(s);
            var r1 = s.getMeanDistance();
            var r;
            if (isFinite(dl) && isFinite(r1)) {
                r = r1/dl;
            } else {
                r = 0;
            }
            
            return (1 + r)*Math.exp(-r);
            // return 1 - 0.6*r
        },
        
        getRpiFactor : function(s)
        {
            var r1 = s.getMeanDistance();
            var q_r = this.r.getQ();
            var q_s = s.getQ();
            if (q_r == 0 || q_s == 0) {
                return 1;
            }
             
            var r_m = s.getMinApproachDistance()*q_r/q_s;
            return Math.exp(-r_m/r1);
        },
        
        getQsHwhm : function(s)
        {
            var n_u = this.n_u;
            var n_l = this.n_l;
            var Z_core = this.r.getQ() + 1;
            var ef = s.getHoltsmarkField();
            var ppi = this.getPpiFactor(s);
            var rpi = this.getRpiFactor(s);
            
            // 1.4385 is S_1 HWHM
            return 1.4385*4.271417e-5*3/2*(n_u*n_u - n_l*n_l)/Z_core*ef*ppi*rpi;
        },
        
        getStarkRatio : function(s)
        {
            var R;
            var w_qs = this.getQsHwhm(s);
            if (w_qs == 0) {
                R = 0;
            } else {
                R = 2*w_qs/(this.getMicrofieldFrequency(s)/3e10/(2*Math.PI));
            }
            
            return R;
        },
        
        getStarkQuasistaticity : function(s)
        {
            var R = this.getStarkRatio(s);
            return R/(R + this.R0);
        },
        
        getStarkHwhm : function()
        {
            var w = 0;
            
            var sa = new Array(this.e, this.i, this.r);
            for (var i = 0; i < 3; i++) {
                var s = sa[i];
                var qs = this.getQsHwhm(s);
                var f  = this.getStarkQuasistaticity(s);

                w += Math.pow(f*qs, 3/2);
            }
            
            return Math.pow(w, 2/3);
        },
        
        getZeemanSplitting : function()
        {
            return 0.466860*this.B;
        },
        
        updateEntityValue : function(entity, species)
        {
            var tree = this.entity_tree;
            var e = tree.getSelectedElement();
            
            if (!e) {
                return;
            }
            
            var entity = e.getUserData("entity");
            var species = e.getUserData("species");
            
            var name = entity.name;
            if (entity.nspecies == 1) {
                switch (species.id) {
                case "e":
                    name += " of electrons";
                    break;
                case "i":
                    name += " of ions";
                    break;
                case "r":
                    name += " of radiators";
                    break;
                }
            }
            this.gui_entity_name.setValue(name);

            this.gui_entity_section.setValue(entity.section);
            
            var formula;
            if (entity.nspecies == 1) {
                var species_str = "this." + species.id;
                formula = entity.formula.replace(/\%s1/g, species_str);
            } else {
                formula = entity.formula;
            }

            var scale = this.gui_entity_units.getSelected().getValue();

            var v = eval(formula);
            this.gui_entity_value.setNumValue(v*scale);
        },

        populateTree : function()
        {
            var i, n = plasma_entities.length;
            var sa = new Array;
            if (this.showElectrons) {
                sa.push(this.e)
            }
            if (this.showIons) {
                sa.push(this.i)
            }
            if (this.showRadiators) {
                sa.push(this.r)
            }
            
            for (var i = 0; i < n; i++) {
                var entity = plasma_entities[i];
                
                if (entity.section == "fundamental" &&
                    this.showFundamental != true) {
                    continue;
                }
                if (entity.section == "spectroscopy" &&
                    this.showSpectroscopy != true) {
                    continue;
                }
                
                var folder;
                switch (entity.dimension) {
                case "frequency":
                    folder = this.gui_folder_freq;
                    break;
                case "length":
                    folder = this.gui_folder_length;
                    break;
                case "velocity":
                    folder = this.gui_folder_vel;
                    break;
                case "energy":
                    folder = this.gui_folder_energy;
                    break;
                case "none":
                    folder = this.gui_folder_none;
                    break;
                default:
                    folder = this.gui_folder_misc;
                    break;
                }
                
                var leaf;
                
                switch (entity.nspecies) {
                case 1:
                    for (var j = 0; j < sa.length; j++) {
                        var s = sa[j];
                        var icon = "pf/image/" + s.id + "_small.png";

                        leaf = new qx.ui.tree.TreeFile(entity.name, icon);
                        leaf.addEventListener("click", clickEntityCB, this);
                        leaf.addEventListener("mouseup", treeMenuCB, this);
                        leaf.setUserData("entity", entity);
                        leaf.setUserData("species", s);

                        folder.add(leaf);
                    }
                    break;
                default:
                    leaf = new qx.ui.tree.TreeFile(entity.name,
                        "pf/image/all_small.png");
                    leaf.addEventListener("click", clickEntityCB, this);
                    leaf.addEventListener("mouseup", treeMenuCB, this);
                    leaf.setUserData("entity", entity);
                    folder.add(leaf);
                    break;
                }
            }
        }, 

        cleanTree : function()
        {
            this.gui_folder_freq.destroyContent();
            
            this.gui_folder_length.destroyContent();
            
            this.gui_folder_vel.destroyContent();
            
            this.gui_folder_none.destroyContent();
            
            this.gui_folder_energy.destroyContent();
            
            this.gui_folder_misc.destroyContent();
        },

        rePopulateTree : function()
        {
            // this.entity_tree.hide();

            this.cleanTree();
            this.populateTree();

            this.gui_folder_freq.close();
            this.gui_folder_length.close();
            this.gui_folder_vel.close();
            this.gui_folder_none.close();
            this.gui_folder_energy.close();
            this.gui_folder_misc.close();

            // this.entity_tree.show();
        },

        /**
         * TODOC
         *
         * @type member
         * @param e {Event} TODOC
         * @return {void}
         */
        main : function(e)
        {
            this.base(arguments);
            
            // Define alias for custom resource path
            qx.io.Alias.getInstance().add("pf",
                qx.core.Setting.get("pf.resourceUri"));

            // Defaults
            
            this.r = new pf.base.Species("r", 1.0,           0, 0,    1.0, null);
            this.e = new pf.base.Species("e", (1.0/1836.2), -1, 1e16, 1.0, this.r);
            this.i = new pf.base.Species("i", 1.0,          +1, 1e16, 1.0, this.r);
            
            this.P_r = 0;

            this.B   = 0;
            
            this.n_u = 2;
            this.n_l = 1;
            
            if (this.updatePlasmaParameters() != true) {
                alert("Error!");
            }
            

            // Top-level container
            this.bl_top = new qx.ui.layout.VerticalBoxLayout("vertical");

            this.bl_top.addToDocument();


            // The toolbar
            var tb = new qx.ui.toolbar.ToolBar;
            this.bl_top.add(tb);

            // ... and the buttons themselves

            var sep = new qx.ui.toolbar.Separator;
            tb.add(sep);

            var btn = new qx.ui.toolbar.Button("About...");
            tb.add(btn);
            // btn.addEventListener("execute", this.aboutCB, this);

            var tf1 = new qx.ui.pageview.tabview.TabView;
            // tf1.setSpacing(0);
            this.bl_top.add(tf1);

            this.tabbar  = tf1.getBar();
            this.tabpane = tf1.getPane();
            this.tabpane.setPadding(0, 0, 0, 0);
            this.tabpane.setBackgroundColor("white");


            var tab_id = new qx.ui.pageview.tabview.Button("Parameters");
            this.tabbar.add(tab_id);
            var page_id = new qx.ui.pageview.tabview.Page(tab_id);
            this.tabpane.add(page_id);

            
            var fr, gl;
            var l, e;

            fr = new qx.ui.groupbox.GroupBox("Electrons",
                "pf/image/e_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#eeeeff");

            page_id.add(fr);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 200);
            gl.setRowCount(2);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);

            fr.add(gl);

            // N_e
            l = new qx.ui.basic.Label("N<sub>e</sub> (cm<sup>-3</sup>):");
            gl.add(l, 0, 0);
            e = new pf.ui.TextField(this.e.getN());
            gl.add(e, 1, 0);
            // e.setLiveUpdate(true);
            e.addEventListener("changeValue", changeN_e, this);

            // T_e
            l = new qx.ui.basic.Label("T<sub>e</sub> (eV):");
            gl.add(l, 0, 1);
            e = new pf.ui.TextField(this.e.getT());
            gl.add(e, 1, 1);
            e.addEventListener("changeValue", changeT_e, this);
            e.setLiveUpdate(true);


            fr = new qx.ui.groupbox.GroupBox("Ions",
                "pf/image/i_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#ffeeee");

            page_id.add(fr);
            fr.setTop(100);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 200);
            gl.setRowCount(4);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);
            gl.setRowHeight(3, 30);

            fr.add(gl);

            // Z_i
            l = new qx.ui.basic.Label("Z<sub>i</sub>:");
            gl.add(l, 0, 0);
            e = new pf.ui.Spinner(1, this.i.getQ(), 100);
            gl.add(e, 1, 0);
            
            e.addEventListener("change", changeZ_i, this);
            
            // M_i
            l = new qx.ui.basic.Label("M<sub>i</sub>:");
            gl.add(l, 0, 1);
            e = new pf.ui.Spinner(1, this.i.getM(), 200);
            gl.add(e, 1, 1);
            e.addEventListener("change", changeM_i, this);
            
            // T_i
            l = new qx.ui.basic.Label("T<sub>i</sub> (eV):");
            gl.add(l, 0, 2);
            e = new pf.ui.TextField(this.i.getT());
            gl.add(e, 1, 2);
            e.addEventListener("changeValue", changeT_i, this);
            e.setLiveUpdate(true);
            this.gui_T_i = e;

            // N_i
            l = new qx.ui.basic.Label("N<sub>i</sub> (cm<sup>-3</sup>):");
            gl.add(l, 0, 3);
            e = new pf.ui.TextField(this.i.getN());
            gl.add(e, 1, 3);
            e.setReadOnly(true);
            this.gui_N_i = e;
            
            
            fr = new qx.ui.groupbox.GroupBox("Radiators",
                "pf/image/r_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#eeffee");

            page_id.add(fr);
            fr.setLeft(330);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 200);
            gl.setRowCount(5);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);
            gl.setRowHeight(3, 30);
            gl.setRowHeight(4, 30);

            fr.add(gl);


            // Z_r
            l = new qx.ui.basic.Label("Z<sub>r</sub>:");
            gl.add(l, 0, 0);
            e = new pf.ui.Spinner(0, this.Z_r, 100);
            gl.add(e, 1, 0);
            e.addEventListener("change", changeZ_r, this);

            // M_r
            l = new qx.ui.basic.Label("M<sub>r</sub>:");
            gl.add(l, 0, 1);
            e = new pf.ui.Spinner(1, this.r.getM(), 200);
            gl.add(e, 1, 1);
            e.addEventListener("change", changeM_r, this);

            // T_r
            l = new qx.ui.basic.Label("T<sub>r</sub> (eV):");
            gl.add(l, 0, 2);
            e = new pf.ui.TextField(this.r.getT());
            gl.add(e, 1, 2);
            e.addEventListener("changeValue", changeT_r, this);
            this.gui_T_r = e;

            // P_r
            l = new qx.ui.basic.Label("P<sub>r</sub> (%):");
            gl.add(l, 0, 3);
            e = new pf.ui.Spinner(0, this.P_r, 100);
            gl.add(e, 1, 3);
            e.addEventListener("change", changeP_r, this);

            // N_r
            l = new qx.ui.basic.Label("N<sub>r</sub> (cm<sup>-3</sup>):");
            gl.add(l, 0, 4);
            e = new pf.ui.TextField(this.r.getN());
            e.setReadOnly(true);
            gl.add(e, 1, 4);
            this.gui_N_r = e;


            fr = new qx.ui.groupbox.GroupBox("Fields");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#ffffdd");

            page_id.add(fr);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 200);
            gl.setRowCount(1);
            gl.setRowHeight(0, 30);

            fr.add(gl);
            fr.setTop(268);


            l = new qx.ui.basic.Label("B (T):");
            gl.add(l, 0, 0);
            e = new pf.ui.TextField(this.B);
            gl.add(e, 1, 0);
            // e.setLiveUpdate(true);
            e.addEventListener("changeValue", changeB, this);

            // Atomic system frame
            fr = new qx.ui.groupbox.GroupBox("Atomic system",
                "pf/image/all_small.png");
            fr.setDimension("auto", "auto");
            fr.setBackgroundColor("#ffffdd");

            page_id.add(fr);

            gl = new qx.ui.layout.GridLayout;
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(2);
            gl.setColumnWidth(0, 100);
            gl.setColumnWidth(1, 200);
            gl.setRowCount(3);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);

            fr.add(gl);
            fr.setLeft(330);
            fr.setTop(202);

            l = new qx.ui.basic.Label("Z<sub>core</sub>:");
            gl.add(l, 0, 0);
            e = new qx.ui.form.TextField(this.r.getQ() + 1);
            e.setReadOnly(true);
            gl.add(e, 1, 0);
            this.gui_Zcore = e;

            l = new qx.ui.basic.Label("n<sub>u</sub>:");
            gl.add(l, 0, 1);
            e = new pf.ui.Spinner(2, this.n_u, 1000);
            gl.add(e, 1, 1);
            e.addEventListener("change", changeN_u, this);

            l = new qx.ui.basic.Label("n<sub>l</sub>:");
            gl.add(l, 0, 2);
            e = new pf.ui.Spinner(1, this.n_l, 1000);
            gl.add(e, 1, 2);
            e.addEventListener("change", changeN_l, this);

            tab_id.setChecked(true);


            var tab_id = new qx.ui.pageview.tabview.Button("Explorer");
            this.tabbar.add(tab_id);
            var page_id = new qx.ui.pageview.tabview.Page(tab_id);
            this.tabpane.add(page_id);
            page_id.setDimension(800, 600);
            page_id.setBackgroundColor("black");
            
            var sp = new qx.ui.splitpane.HorizontalSplitPane(300, "1*");
            sp.set(
            {
                left            : 5,
                right           : 5,
                top             : 5,
                bottom          : 5,
                border          : "inset-thin",
                backgroundColor : "white",
                showKnob        : true
            });

            var fff = new qx.ui.layout.CanvasLayout();
            fff.set(
            {
                width:  800,
                height: 600
            });
            
            page_id.add(fff);
            fff.add(sp);
                        
            var left_pane  = sp.getLeftArea();
            left_pane.setPaddingRight(4);
            var right_pane = sp.getRightArea();
            right_pane.setPaddingLeft(4);
            var tree = new qx.ui.tree.Tree("Browser");
            tree.set(
            {
                top             : 4,
                left            : 4,
                bottom          : 4,
                backgroundColor : "white",
                border          : "inset-thin",
                overflow        : "scroll",
                height          : "100%",
                width           : "100%",
                paddingLeft     : 4,
                paddingTop      : 4
            });
            left_pane.add(tree);
            
            this.entity_tree = tree;
            
            var folder;
            
            folder = new qx.ui.tree.TreeFolder("Frequencies");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_freq = folder;

            folder = new qx.ui.tree.TreeFolder("Lengths");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_length = folder;
            
            folder = new qx.ui.tree.TreeFolder("Velocities");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_vel = folder;
            
            folder = new qx.ui.tree.TreeFolder("Dimensionless");
            tree.add(folder);
            this.gui_folder_none = folder;

            folder = new qx.ui.tree.TreeFolder("Energies");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_energy = folder;

            folder = new qx.ui.tree.TreeFolder("Miscellaneous");
            folder.setAlwaysShowPlusMinusSymbol(true);
            tree.add(folder);
            this.gui_folder_misc = folder;

            this.populateTree();

            tree.setHideNode(true);
            tree.setRootOpenClose(true);
            
            // Build context-sensitive popup menu
            this.treeMenu = new qx.ui.menu.Menu;
            var button;
            
            var subMenu1 = new qx.ui.menu.Menu;
            button = new qx.ui.menu.CheckBox("Fundamental",
                null, this.showFundamental);
            button.addEventListener("execute", showFundamentalCB, this);
            subMenu1.add(button);
            button = new qx.ui.menu.CheckBox("Spectroscopy",
                null, this.showSpectroscopy);
            button.addEventListener("execute", showSpectroscopyCB, this);
            subMenu1.add(button);
            this.treeMenu.add(new qx.ui.menu.Button("Show sections",
                null, null, subMenu1));
            
            this.treeMenu.add(new qx.ui.menu.Separator);

            var subMenu2 = new qx.ui.menu.Menu;
            button = new qx.ui.menu.CheckBox("Electrons",
                null, this.showElectrons)
            button.addEventListener("execute", showElectronsCB, this);
            subMenu2.add(button);
            button = new qx.ui.menu.CheckBox("Ions",
                null, this.showIons)
            button.addEventListener("execute", showIonsCB, this);
            subMenu2.add(button);
            button = new qx.ui.menu.CheckBox("Radiators",
                null, this.showRadiators)
            button.addEventListener("execute", showRadiatorsCB, this);
            subMenu2.add(button);
            this.treeMenu.add(new qx.ui.menu.Button("Show species",
                null, null, subMenu2));
            
            this.treeMenu.add(new qx.ui.menu.Separator);

            button = new qx.ui.menu.Button("Add to favorites");
            button.addEventListener("execute", addToFavCB, this);
            this.treeMenu.add(button);
            this.treeMenuButtonAddToFav = button;
            button = new qx.ui.menu.Button("Remove from favorites");
            button.addEventListener("execute", removeFromFavCB, this);
            this.treeMenu.add(button);
            this.treeMenuButtonRemoveFromFav = button;
            
            var doc = qx.ui.core.ClientDocument.getInstance();
            doc.add(this.treeMenu, subMenu1, subMenu2);


            gl = new qx.ui.layout.GridLayout;
            gl.set(
            {
                left:   4,
                right:  4,
                top:    4
            });
            
            gl.setVerticalSpacing(4);
            gl.setHorizontalSpacing(6);
            gl.setColumnCount(3);
            gl.setColumnWidth(0, "15%");
            gl.setColumnWidth(1, "60%");
            gl.setColumnWidth(2, "20%");
            gl.setRowCount(3);
            gl.setRowHeight(0, 30);
            gl.setRowHeight(1, 30);
            gl.setRowHeight(2, 30);

            l = new qx.ui.basic.Label("Name:");
            gl.add(l, 0, 0);
            e = new qx.ui.form.TextField();
            e.setReadOnly(true);
            this.gui_entity_name = e;
            gl.add(e, 1, 0);

            l = new qx.ui.basic.Label("Value:");
            gl.add(l, 0, 1);
            e = new pf.ui.TextField("");
            e.setReadOnly(true);
            gl.add(e, 1, 1);
            this.gui_entity_value = e;

            e = new pf.ui.ComboBox();
            e.addEventListener("changeSelected", changeUnitsCB, this);
            gl.add(e, 2, 1);
            this.gui_entity_units = e;

            l = new qx.ui.basic.Label("Section:");
            gl.add(l, 0, 2);
            e = new qx.ui.form.TextField();
            e.setReadOnly(true);
            this.gui_entity_section = e;
            gl.add(e, 1, 2);

            right_pane.add(gl);
        },
    

        /**
         * TODOC
         *
         * @type member
         * @param e {Event} TODOC
         * @return {void}
         */
        close : function(e)
        {
            this.base(arguments);

            // Prompt user
            // return "qooxdoo application: Do you really want to close the application?";
        },


       /**
        * TODOC
        *
        * @type member
        * @param e {Event} TODOC
        * @return {void}
        */
        terminate : function(e) {
            this.base(arguments);
        }
    },

    /*
    *****************************************************************************
       SETTINGS
    *****************************************************************************
    */

    settings : {
        "pf.resourceUri" : "./resource"
    }
});
